from sinta_scraper.affil_scraper import affil, affil_authors
from sinta_scraper.author_scraper import author
from sinta_scraper.comm_svc_scraper import author_comm_services, dept_comm_services
from sinta_scraper.dept_scraper import dept_authors
from sinta_scraper.ipr_scraper import author_ipr, dept_ipr
from sinta_scraper.research_scraper import author_researches, dept_researches
from sinta_scraper.scholar_documents_scraper import author_scholar
from sinta_scraper.scopus_documents_scraper import author_scopus, author_scopus_journal, \
    author_scopus_conference, dept_scopus
from sinta_scraper.wos_documents_scraper import author_wos, dept_wos
